import { Injectable } from '@nestjs/common';
import { CreateDataSchedulingManager } from './managers/create-data-scheduling.manager';
import { DataSchedulingDataService } from '../../data/services/data-scheduling-data.service';
import { DataSchedulingEntity } from '../entities/data-scheduling.entity';
import { DeleteDataSchedulingManager } from './managers/delete-data-scheduling.manager';
import { UpdateDataSchedulingManager } from './managers/update-data-scheduling.manager';
import { BaseDataTransactionOrchestrator } from 'src/core/modules/domain/usecase/orchestrators/base-data-transaction.orchestrator';
import { ActiveDataSchedulingManager } from './managers/active-data-scheduling.manager';
import { InactiveDataSchedulingManager } from './managers/inactive-data-scheduling.manager';
import { ConfirmDataSchedulingManager } from './managers/confirm-data-scheduling.manager';
import { STATUS } from 'src/core/strings/constants/base.constants';
import { BatchResult } from 'src/core/response/domain/ok-response.interface';
import { BatchConfirmDataSchedulingManager } from './managers/batch-confirm-data-scheduling.manager';
import { BatchInactiveDataSchedulingManager } from './managers/batch-inactive-data-scheduling.manager';
import { BatchActiveDataSchedulingManager } from './managers/batch-active-data-scheduling.manager';
import { BatchDeleteDataSchedulingManager } from './managers/batch-delete-data-scheduling.manager';
import { TABLE_NAME } from 'src/core/strings/constants/table.constants';

@Injectable()
export class DataSchedulingDataOrchestrator extends BaseDataTransactionOrchestrator<DataSchedulingEntity> {
  constructor(
    private createManager: CreateDataSchedulingManager,
    private updateManager: UpdateDataSchedulingManager,
    private deleteManager: DeleteDataSchedulingManager,
    private activeManager: ActiveDataSchedulingManager,
    private confirmManager: ConfirmDataSchedulingManager,
    private inactiveManager: InactiveDataSchedulingManager,
    private batchDeleteManager: BatchDeleteDataSchedulingManager,
    private batchActiveManager: BatchActiveDataSchedulingManager,
    private batchConfirmManager: BatchConfirmDataSchedulingManager,
    private batchInactiveManager: BatchInactiveDataSchedulingManager,
    private serviceData: DataSchedulingDataService,
  ) {
    super();
  }

  async create(data): Promise<DataSchedulingEntity> {
    this.createManager.setData(data);
    this.createManager.setService(this.serviceData, TABLE_NAME.DATA_SCHEDULING);
    await this.createManager.execute();
    await this.createManager.generateConfig();
    return this.createManager.getResult();
  }

  async update(dataId, data): Promise<DataSchedulingEntity> {
    this.updateManager.setData(dataId, data);
    this.updateManager.setService(this.serviceData, TABLE_NAME.DATA_SCHEDULING);
    await this.updateManager.execute();
    return this.updateManager.getResult();
  }

  async delete(dataId): Promise<string> {
    this.deleteManager.setData(dataId);
    this.deleteManager.setService(this.serviceData, TABLE_NAME.DATA_SCHEDULING);
    await this.deleteManager.execute();
    return this.deleteManager.getResult();
  }

  async batchDelete(dataIds: string[]): Promise<BatchResult> {
    this.batchDeleteManager.setData(dataIds);
    this.batchDeleteManager.setService(
      this.serviceData,
      TABLE_NAME.DATA_SCHEDULING,
    );
    await this.batchDeleteManager.execute();
    return this.batchDeleteManager.getResult();
  }

  async active(dataId): Promise<string> {
    this.activeManager.setData(dataId, STATUS.ACTIVE);
    this.activeManager.setService(this.serviceData, TABLE_NAME.DATA_SCHEDULING);
    await this.activeManager.execute();
    return this.activeManager.getResult();
  }

  async batchActive(dataIds: string[]): Promise<BatchResult> {
    this.batchActiveManager.setData(dataIds, STATUS.ACTIVE);
    this.batchActiveManager.setService(
      this.serviceData,
      TABLE_NAME.DATA_SCHEDULING,
    );
    await this.batchActiveManager.execute();
    return this.batchActiveManager.getResult();
  }

  async confirm(dataId): Promise<string> {
    this.confirmManager.setData(dataId, STATUS.ACTIVE);
    this.confirmManager.setService(
      this.serviceData,
      TABLE_NAME.DATA_SCHEDULING,
    );
    await this.confirmManager.execute();
    return this.confirmManager.getResult();
  }

  async batchConfirm(dataIds: string[]): Promise<BatchResult> {
    this.batchConfirmManager.setData(dataIds, STATUS.ACTIVE);
    this.batchConfirmManager.setService(
      this.serviceData,
      TABLE_NAME.DATA_SCHEDULING,
    );
    await this.batchConfirmManager.execute();
    return this.batchConfirmManager.getResult();
  }

  async inactive(dataId): Promise<string> {
    this.inactiveManager.setData(dataId, STATUS.INACTIVE);
    this.inactiveManager.setService(
      this.serviceData,
      TABLE_NAME.DATA_SCHEDULING,
    );
    await this.inactiveManager.execute();
    return this.inactiveManager.getResult();
  }

  async batchInactive(dataIds: string[]): Promise<BatchResult> {
    this.batchInactiveManager.setData(dataIds, STATUS.INACTIVE);
    this.batchInactiveManager.setService(
      this.serviceData,
      TABLE_NAME.DATA_SCHEDULING,
    );
    await this.batchInactiveManager.execute();
    return this.batchInactiveManager.getResult();
  }
}
